/*
FileName:		BUFR_StructsConsts.h
Author:			Lewis, O
Description:	Const definiton header file for WMO BUFR encoding
Copyright (c)  2017 Sutron Corp
*/
#include <stddef.h>   


#ifndef _BUFR_STRUCTSCONSTS_H__
#define _BUFR_STRUCTSCONSTS_H__
#include "TosTypes.h"

//this is used for the length field common to all sections
#define LEN_BUFF_BYTE_CNT	3
#define LEN_ORIGIN_CENTRE	2


//table version numbers 
#define MASTER_TABLE_VER			28
#define BUFR_MASTER_TABLE_VERSION	0

#define VAL_SECTION_5				'7'
 

typedef struct
{
	uint8_t	acBUFR[4];//"BUFR" string
	uint8_t	acLen[3];
	uint8_t	cBUFRedition;//currently edition 2
}STRUCT_SECTION_0;
enum eDATA_CATEGORY
{
	eSURFACE_DATA_LAND,
	eSURFACE_DATA_SEA,
	eVERTICALSURROUNDINGS_NOT_SAT,
	eVERTICALSURROUNDINGS_SAT,
	eSINGLE_LEVEL_UPPER_AIR_DATA,

};
typedef struct
{
	uint8_t cOptSectIncluded:1;// 0 = no optional section (#2), 1 = opt section included
	uint8_t cReserved:7;
}STRUCT_OPT_SECT_INCL;
typedef struct
{
	unsigned char			acLen[LEN_BUFF_BYTE_CNT];
	uint8_t					cMasterTableBUFR;//0 for WMO, 10 for IOC, etc 
	uint8_t					acOrigCentre[LEN_ORIGIN_CENTRE];	
	uint8_t					cUpdateSeqNum;//zero for original BUFRmessages; incremented for updates
	STRUCT_OPT_SECT_INCL	strctOptionalSection;//Bit 1 = 0 No optional section= 1 Optional section included
	uint8_t					cDataCatType;//(BUFR Table A)
	uint8_t					cDataCatSubType;
	uint8_t					cMasterTableVer;//curr 9 for WMO 94 tables
	uint8_t					cLocalTableVer;
	uint8_t					cYear;
	uint8_t					cMonth;
	uint8_t					cDay;
	uint8_t					cHour;
	uint8_t					cMin;
	uint8_t					cReserved;
}STRUCT_SECTION_1;

 
typedef struct
{
	uint8_t	acLen[LEN_BUFF_BYTE_CNT];
	uint8_t	cReserved1;//set to zero
	uint8_t	cReserved2;//set to zero
}STRUCT_SECTION_2;
 

typedef struct
{	
	uint8_t cX:6;
	uint8_t cF:2;	
	uint8_t cY:8;
	
}STRUCT_TABLE_B_REF;

typedef union
{
	STRUCT_TABLE_B_REF	strctRef;
	uint16_t			iTableBRef;
}UNION_TABLE_B_REF;

#define NUM_DATA_SUBSETS_BYTES		2
enum
{
	eOTHER_DATA =0,
	eOBSERVED_DATA ,	
};
enum
{
	eNON_COMPRESSED_DATA = 0,
	eCOMPRESSED_DATA,
};
typedef struct
{
	uint8_t cReserved:6;	
	uint8_t cCompressedDataFlag:1;
	uint8_t cObservedDataFlag:1;
}STRUCT_DATA_FLAGS;

typedef struct
{
	uint8_t				acLen[LEN_BUFF_BYTE_CNT];
	uint8_t				cReserved;//set to zero
	uint8_t				acNumDataSubsets[NUM_DATA_SUBSETS_BYTES];
	STRUCT_DATA_FLAGS	strctDataFlags;
	STRUCT_TABLE_B_REF*	ptrStrctArrRefDescr;
}STRUCT_SECTION_3;
#define SECTION3_HDR_BYTE_CNT	offsetof(STRUCT_SECTION_3, ptrStrctArrRefDescr)


typedef struct
{
	uint8_t		acLen[LEN_BUFF_BYTE_CNT];
	uint8_t		cReserved;//set to zero
	uint8_t*	acPtrData;
	uint32_t	lNumBitsUsed;//Only for internal use . used to indicate wether a padding byte was added
}STRUCT_SECTION_4;

#define SECTION4_HDR_BYTE_CNT	offsetof(STRUCT_SECTION_4, acPtrData)
typedef struct
{
	uint8_t	acTerminateStr[4];//Set to "7777"
}STRUCT_SECTION_5;
 
 
 
typedef struct
{
	STRUCT_TABLE_B_REF	strctRef;
	const char*			acDescription; 
	const char*			acPtrUnitsString;//place holder : not used
	int16_t				iScale;
	int32_t				lRefVal;
	int8_t				iDataBitWidth;
}STRUCT_TABLE_B_ELEMENT;

 	
 
typedef struct 
{
	uint16_t	iYear;
	uint8_t		cMonth;
	uint8_t		cDay;
	uint8_t		cHour;
	uint8_t		cMin;
}STRUCT_DATE_TIME;

typedef struct
{
	double					dVal;
	STRUCT_TABLE_B_REF		strctTableBRefDesc;
}STRUCT_VAL_REF_DESC;
#endif